package com.jqtx.windows.common.config;

import com.alibaba.csp.sentinel.adapter.spring.webmvc.callback.BlockExceptionHandler;
import com.alibaba.csp.sentinel.slots.block.BlockException;
import com.alibaba.csp.sentinel.slots.block.authority.AuthorityException;
import com.alibaba.csp.sentinel.slots.block.degrade.DegradeException;
import com.alibaba.csp.sentinel.slots.block.flow.param.ParamFlowException;
import com.alibaba.csp.sentinel.slots.system.SystemBlockException;
import com.alibaba.fastjson.JSON;
import com.jqtx.windows.infrastructure.enums.ExceptionCodeEnum;
import org.springframework.context.annotation.Configuration;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@Configuration
public class SentinelConfig implements BlockExceptionHandler {

    @Override
    public void handle(HttpServletRequest request, HttpServletResponse response, BlockException e) throws Exception {
        // BlockException 异常接口，其子类为Sentinel五种规则异常的实现类
        // AuthorityException 授权异常
        // DegradeException 降级异常
        // FlowException 限流异常
        // ParamFlowException 参数限流异常
        // SystemBlockException 系统负载异常
        ExceptionCodeEnum codeEnum = ExceptionCodeEnum.CURRENT_LIMIT_ERROR;
        if (e instanceof DegradeException) {
            codeEnum = ExceptionCodeEnum.SERVICE_DEMO_ERROR;
        } else if (e instanceof ParamFlowException) {
            codeEnum = ExceptionCodeEnum.HOT_CHAR_ERROR;
        } else if (e instanceof SystemBlockException) {
            codeEnum = ExceptionCodeEnum.SYSTEM_FAIL_ERROR;
        } else if (e instanceof AuthorityException) {
            codeEnum = ExceptionCodeEnum.ROLE_FAIL_ERROR;
        }
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json");
        response.getWriter().write(JSON.toJSONString(JsonResult.error(codeEnum)));
    }
}